"""
number.py
written in Python3
author: C. Lockhart <chris@lockhartlab.org>
"""

from .any import AnyLike

from abc import ABCMeta
import numpy as np


# NumberLike class
class NumberLike(AnyLike, metaclass=ABCMeta):
    """
    Something that is ``NumberLike`` is something that is singular and numerical

    Note: this class is not implemented. Don't create an instance, because it doesn't do anything.
    """

    # Needed to trick PyCHarm
    # noinspection PyMissingConstructor
    def __init__(self):
        raise NotImplementedError


# Register subclasses
NumberLike.register(int)
NumberLike.register(float)
NumberLike.register(bool)
NumberLike.register(np.int)
NumberLike.register(np.float)
NumberLike.register(np.bool)



"""
pandas.py
written in Python3
author: C. Lockhart <chris@lockhartlab.org>
"""

from .any import AnyLike

from abc import ABCMeta
import pandas as pd


# Pandas class
class Pandas(AnyLike, metaclass=ABCMeta):
    """
    ``Pandas`` is pandas.Series and pandas.DataFrame

    Note: this class is not implemented. Don't create an instance, because it doesn't do anything.
    """

    # Needed to trick PyCharm
    # noinspection PyMissingConstructor
    def __init__(self):
        self.shape = None
        raise NotImplementedError


# Register subclasses
Pandas.register(pd.DataFrame)
Pandas.register(pd.Series)

AnyLike.register(Pandas)




from .core import *
from .version import __version__

# __all__ = [
#     'core',
#     '__version__'
# ]

__all__ = [
    'AnyLike',
    'Anything',
    'ListLike',
    'NumberLike',
    'typelike',
    'Undefined',
    '__version__'
]



"""
any.py
written in Python3
author: C. Lockhart <chris@lockhartlab.org>
"""

from abc import ABCMeta

__all__ = [
    'AnyLike',
    'Anything'
]


# AnyLike class
class AnyLike(metaclass=ABCMeta):
    """
    Anything is ``AnyLike``

    Note: this class is not implemented. Don't create an instance, because it doesn't do anything.
    """

    # Needed to trick PyCharm
    def __init__(self):
        raise NotImplementedError

    # Needed to trick PyCharm
    def __getitem__(self, item):
        raise NotImplementedError

    # Needed to trick PyCharm
    def __iter__(self):
        raise NotImplementedError

    # Needed to trick PyCharm
    def __len__(self):
        raise NotImplementedError

    # Register subclass as ArrayLike
    @classmethod
    def register(cls, subclass):
        """
        Registers a new subclass

        Parameters
        ----------
        subclass : class
            Subclass to register
        """

        # noinspection PyCallByClass
        ABCMeta.register(cls, subclass)


# Register subclasses
# AnyLike.register()

# Since `AnyLike` doesn't really make sense, let's alias this as `Anything`
Anything = AnyLike




from .any import *
from .core import *
from .list import *
from .none import *
from .number import *

__all__ = [
    'AnyLike',
    'Anything',
    'infer_type',
    'ListLike',
    'NoneType',
    'NumberLike',
    'Undefined'
]



"""
typelike.py
written in Python3
author: C. Lockhart <chris@lockhartlab.org>
"""

import numpy as np
import pandas as pd


# A list of types
dtypes = {
    'int': int,
    'float': float,
    'str': str,
    'list': list,
    'tuple': tuple,
    'set': set,
    'dict': dict,
    'numpy': np.ndarray,
    'series': pd.Series,
    'dataframe': pd.DataFrame
}


def infer_type(anything, itemize=True):
    """
    Infer the type of anything

    Applies some logic to parse anything

    Parameters
    ----------
    anything : Anything
        Something of whose type we need to infer
    itemize : bool
        Should we infer the items in lists, tuples? (Default: True)

    Returns
    -------
    object
        Type of `anything`
    """

    # By default, set the type to the type of anything
    dtype = type(anything)

    # If anything is a string, try to parse
    if isinstance(anything, str):
        dtype = dtypes.get(anything.lower(), dtype)

    # If anything is a tuple or list, parse each element individually
    elif itemize and isinstance(anything, (list, tuple)):
        dtype = []
        for item in anything:
            dtype.append(infer_type(item, itemize=True))
        if isinstance(anything, tuple):
            dtype = tuple(dtype)

    # Return
    return dtype



"""
list.py
written in Python3
author: C. Lockhart <chris@lockhartlab.org>
"""

from .any import AnyLike

from abc import ABCMeta
import numpy as np
import pandas as pd


# ListLike class
class ListLike(AnyLike, metaclass=ABCMeta):
    """
    Something that is ``ListLike`` is something that can be coerced into a 1-dimensional list. This includes lists,
    sets, tuples, numpy.ndarray, and pandas.Series.

    Note: this class is not implemented. Don't create an instance, because it doesn't do anything.
    """

    # Needed to trick PyCharm
    # noinspection PyMissingConstructor
    def __init__(self):
        self.shape = None
        raise NotImplementedError


# Register subclasses
ListLike.register(list)
ListLike.register(set)
ListLike.register(tuple)
ListLike.register(np.ndarray)  # TODO is there a way to specify only 1D numpy arrays?
ListLike.register(pd.Series)

AnyLike.register(ListLike)
